#ifndef __TCPSOCKET_H
#define __TCPSOCKET_H

#include "recvform.h"
#include "sendform.h"
#include "crypto.h"

#include <qstringlist.h>
#include <qsocket.h>
#include <qfile.h>

#define TCP_PROTOCOL "vychat"
#define TCP_VERSION "1"
#define TCP_FILE_TRANSFER_READY "file-transfer-ready"
#define TCP_FILE_TRANSFER "file-transfer"
#define TCP_FILE_TRANSFER_DONE "file-transfer-done"
#define TCP_FILE_TRANSFER_DENY "file-transfer-deny"
#define TCP_AVATAR_TRANSFER "avatar-transfer"
#define TCP_AVATAR_TRANSFER_READY "avatar-transfer-ready"
#define TCP_BOARD_REQUEST "board-request"
#define TCP_BINARY_TYPE "binary"
#define TCP_CHAT_REQUEST "chat-request"
#define TCP_CHAT_REQUEST_OK "chat-request-ok"
#define TCP_CHAT_MSG_TYPING "chat-msg-typing"
#define TCP_CHAT_MSG "chat-msg"
#define TCP_PVTMSG "pvtmsg"
#define TCP_PVTMSG_CONFIRM "pvtmsg-confirm"
#define TCP_PING "ping"
#define TCP_REQUEST_MISSED "request-missed"
#define HTTP_GET "GET"
#define HTTP_POST "POST"

#define SENDING_BLOCK 8192

enum
{
    PROTOCOL_ID,
    VERSION_ID,
    MESSAGE_TYPE_ID,
    MESSAGES_ID,
    CONTENT_TYPE_ID,
    CONTENT_LENGTH_ID,
    FILE_NAME_ID,
    FILE_DESC_ID,
    FILES_LEFT_ID,
    DESC_FILES_NAME_ID,
    DESC_FILES_SIZE_ID,
    USER_NAME,
    USER_UUID_ID,
    AUTO_ANSWER_ID,
    USER_STATUS_ID,
    PVT_MESSAGE_SIG_ID,
    PVT_MASS_ID,
    DATA_SIGNATURE_ID,
    CKEY_ID,
    TYPING_MODE_ID,
    HTTP_CONTENT_LENGTH_ID,
    HTTP_CONTENT_TYPE_ID
};

const struct
{
    const char *varname;
    char id;
} varspack[] = {
    {"protocol", PROTOCOL_ID},
    {"version", VERSION_ID},
    {"message-type", MESSAGE_TYPE_ID},
    {"messages-id", MESSAGES_ID},
    {"content-type", CONTENT_TYPE_ID},
    {"content-length", CONTENT_LENGTH_ID},
    {"file-name", FILE_NAME_ID},
    {"file-desc", FILE_DESC_ID},
    {"files-left", FILES_LEFT_ID},
    {"desc-files-name", DESC_FILES_NAME_ID},
    {"desc-files-size", DESC_FILES_SIZE_ID},
    {"user-name", USER_NAME},
    {"user-uuid", USER_UUID_ID},
    {"auto-answer", AUTO_ANSWER_ID},
    {"user-status", USER_STATUS_ID},
    {"pvt-message-sig", PVT_MESSAGE_SIG_ID},
    {"pvt-mass", PVT_MASS_ID},
    {"data-signature", DATA_SIGNATURE_ID},
    {"ckey", CKEY_ID},
    {"typing-mode", TYPING_MODE_ID},
    {"Content-Length", HTTP_CONTENT_LENGTH_ID},
    {"Content-Type", HTTP_CONTENT_TYPE_ID},
    {'\0', 0}
};

const struct
{
    const char *ext;
    const char *mime;
} mimepack[] = {
    {"unknowntype", "application/unknown"},
    {"avi", "video/avi"},
    {"bmp", "image/x-MS-bmp"},
    {"dat", "application/raw"},
    {"com", "application/octet-stream"},
    {"doc", "application/msword"},
    {"dtd", "text/xml"},
    {"exe", "application/octet-stream"},
    {"gif", "image/gif"},
    {"htm", "text/html"},
    {"html", "text/html"},
    {"jpg", "image/jpeg"},
    {"jpeg", "image/jpeg"},
    {"mid", "audio/midi"},
    {"midi", "audio/midi"},
    {"mov", "video/quicktime"},
    {"mpe", "video/mpeg"},
    {"mpeg", "video/mpeg"},
    {"mpg", "video/mpeg"},
    {"mp2", "audio/x-mpeg"},
    {"mp3", "audio/x-mpeg"},
    {"pcd", "image/x-photo-cd"},
    {"pct", "image/pict"},
    {"pcx", "image/pcx"},
    {"pdf", "application/pdf"},
    {"php", "text/plain"},
    {"php3", "text/plain"},
    {"pic", "image/pict"},
    {"pict", "image/pict"},
    {"png",  "image/png"},
    {"tar", "application/x-tar"},
    {"tif", "image/tiff"},
    {"tgz", "application/x-gtar"},
    {"tif", "image/tiff"},
    {"tiff", "image/tiff"},
    {"txt", "text/plain"},
    {"url", "application/internet-shortcut"},
    {"wav", "audio/x-wav"},
    {"xls", "application/vnd.ms-excel"},
    {"xml", "text/xml"},
    {"z", "application/x-compress"},
    {"zip", "application/x-zip-compressed"},
    {'\0', '\0'}
};

class TCPSocket : public QSocket
{
    Q_OBJECT

public:
    TCPSocket( QObject *parent, CRYPTKEY *pCryptKey);
    ~TCPSocket();
    
    void createChatSession(const QHostAddress &addr, Q_UINT16 port, const QString &uid, CRYPTKEY *pubkey);
    bool isChatSession() { return chatsession; }
    int version() { return vers; }
    Q_ULONG contentLength() { return contentlength; }
    Q_ULONG filesLeft() { return filesleft; }
    Q_ULONG descFilesSize() { return descfilessize; }
    const QString &protocol() { return prot; }
    const QString &messageType() { return messagetype; }
    const QString &contentType() { return contenttype; }
    const QString &fileName() { return filename; }
    const QString &fileDesc() { return filedesc; }
    const QString &descFilesName() { return descfilesname; }
    const QString &userName() { return username; }
    const QString &userUuid() { return useruuid; }
    const QString &getSessUid() { return sessuid; }
   
    void httpGet(const QString& fn, const QHostAddress& addr, Q_UINT16 port);
    void sendFile(const QStringList& filesnames, const QString& descr, const QString& recv, const QString& uuid, const QHostAddress& addr, Q_UINT16 port, bool avatar);
    void boardMsgRequest(const QHostAddress& addr, const QString& nick, const QString& uid, Q_UINT32 id, Q_UINT16 port);
    void sendMessageACK(const QHostAddress& addr, const QString& nick, const QString& uid, char status, const QString& answer, const QCString& sig, Q_UINT16 port);
    void sendMessage(const QHostAddress& addr, const QString& nick, const QString& uid, const QString &text, Q_UINT16 port);
    void sendChatMessage(const QString &text);
    void sendTypingMode(uint tmode);

signals:

    void closeConnection();
    void fileRecvFinished(const QString&, const QString&);
    void fileSendFinished(const QString&, const QString&);
    void pvtMsgReceived(const QHostAddress &, const QString &, const QString &, const QString &, const QString &, const QString &);
    void messageACKReceived(const QHostAddress&, const QString&, const QString &, const QString&, char);
    void pingReceived(const QString&, const QString&);
    void boardRequestMissedReceived(const QString&, const QString&, const QString&);
    void recvChatMessage(const QHostAddress&, const QString&);
    void chatSessionEstablishedByRemote(const QString&);
    void chatSessionEstablished(const QString&);
    void recvTypingMode(const QString&, uint);
    
private slots:
    void read();
    void connectDone();
    void closeConn();
    void writtenBlock(int);
    void checkConnection();
    void acceptFile(const QString& sender, const QString& fpath, bool avatar);
    void fileTransferDeny(int startfrom);

private:
    bool vis;
    bool chatsession;
    bool initialized;
    bool accepted;
    bool connectestablished;
    bool sessionestablished;
    bool httpsession;
    int vers;
    int userstatus;
    uint typingmode;
    Q_ULONG contentlength;
    Q_ULONG filesleft;
    Q_ULONG receivedbytes;
    Q_ULONG sendbytes;
    Q_ULONG descfilessize;
    Q_ULONG filenumber;
    Q_ULONG totalfiles;
    Q_ULONG currfilenum;
    Q_ULONG updlim;
    Q_ULONG currtransfer;
    QCString pvtmessagesig;
    QCString ckey;
    QString prot;
    QString messagetype;
    QString contenttype;
    QString filename;
    QString fullpath;
    QString filedesc;
    QString descfilesname;
    QString useruuid;
    QString sessuid;
    QString remoteusername;
    QString username;
    QString filespath;
    QString sendtype;
    QString messagesid;
    QString autoanswer;
    QString pvtmass;
    QString datasignature;
    QString messages;
    QString text;
    QStringList filesnames;
    
    
    QStringList packet;
    QFile file;
    RecvForm* rf;
    SendForm* sf;
    
    CRYPTKEY *pkey;
    CRYPTKEY *skey;
    
    QString ext2mime(const QString&ext);
    QString mime2ext(const QString&mime);
    void sendpacket();
    void acceptMessage(const QByteArray&ba);
    void fileReadError();
    void writeNextBlock();
    void sendNextFile();
    void httpError();
    
//    void sessionEstablishingError();
};

#endif
